/****** PiTroepfel.c ***********************************************
*
*   PROGRAM
*      PiTroepfel
*
*   CONTENTS
*      A function that calculates PI Step by step an sends result to
*      a stream. If the stream is not given, its send to stdout.
*
*   AUTHOR
*      Georg Pfundt
*
*   COPYRIGHT
*       by Georg Pfundt 03-JAN-1996  11:34:41
*
*   LANGUAGE
*      ANSI-C
*
*   TRANSLATOR
*      SAS-C V6.55
*
*   HISTORY
*      1.0, GeP, 03-JAN-1996, first version
*      <Version, Autor, Datum, Bemerkung>
*
*   SUPPORT
*      Literature:
*
*      Spektrum der Wissenschaft, Stewart, Ian; n12 a1995 pp10-14
*
*      "A Spigot Algorithm for the Digits of Pi", Rabinowitz, Stanley;
*      Wagon, Stan; American Mathematical Monthly, v102 n3 a1995 pp195-203
*
*      "The Calculation of e to many Significant Digits", Sale, A. H. J.;
*      Computing Journal, v11 a1968 pp229-230
*
*   IMPORTS
*      None.
*
*   BUGS
*      None known.
*
*   ADDRESS
*      Georg Pfundt, Im Oberviertel 18, 76229 Karlsruhe, Germany
*
*   PHONE
*      GERMANY - (0)721/481591
*      E-Mail: PFUNDT@PHYSIK.UNI-KARLSRUHE.DE
*          or: gp@ict.fhg.de  (till 30-APR-1996)
*
*   UPDATE
*
*   USAGE
*      long PiTroepfel(long sig, FILE *out);
*
*      sig - number of significant places to compute.
*      out - a C-filehandle or NULL for stdout.
*
*      result - number of calculated digits, might by a few less than
*               sig.
*
********************************************************************
* TABSIZE=2
*/

#include <stdio.h>
#include <stdlib.h>
 /* #include <string.h> */



/* memory allocation function for amiga because of limitations with malloc
 AND shortints */
#ifdef _SHORTINT
#include <proto/exec.h>
#define ALLOCMEM(x) AllocVec((ULONG)(x),NULL)
#define FREEMEM(x) FreeVec(x)
#else
/* for other(=unix) systems */
#define ALLOCMEM(x) malloc((size_t)(x))
#define FREEMEM(x) free(x)
#endif

/* check for amiga */
#ifndef _AMIGA
#define __aligned
#endif


/* type of digits*/
typedef unsigned long digit_t;

/* size of buffer for prelimnary digits */
#define PRESTACK 16


__aligned long
PiTroepfel(long sig,FILE *out)
{
	long ndig,h,h1,d,i,j;
	digit_t *dig,*ptr;
	digit_t stack[PRESTACK];
  int stkptr=0;

	/* set output to stdout if not given */
	if (out == NULL)
		out=stdout;

	/* increment number of significant digits by 1 to get sig digits result */
	sig++;

	/* number of digits we need*/
	ndig = (10L*sig)/3L;

	/* alloc mem for digits */
	if ((dig = (digit_t*)ALLOCMEM(ndig*sizeof(digit_t))) == NULL)
	{
		printf ("***Error: No memory (%ld Bytes)!\n\n",ndig*sizeof(digit_t));
		return 0;
	}

	/* fill mem with 2 */
	i=ndig;
	ptr=dig;
	while(i--)
		*(ptr++)=2;


	/* start algo */
	for (i=0 ; i<sig ; i++)
	{
		/* multiply by 10 */
		ptr=dig+ndig-1;
		if (*ptr > (0xffffffff/10L))
		{
			printf ("***Error: Overflow1!\n");
			break;
		}
		*ptr *= 10;

		j=ndig;
		while(--j > 0)
		{
			h=(long)(*ptr);
			d=2*j+1;
      *ptr = h % d;
      h1=(long)(*(--ptr));
      h1 = h1*10 + (h / d)*j;
      if (h1 < 0)
			{
				printf ("\n***Error: Overflow2 at i=%ld, j=%ld, h1=%ld!, r=%lu,"
								"h=%ld, d=%ld\n"
								,i,j,h1,(unsigned long)(*ptr),h,d);
				break;
			}
      *ptr = h1;
		}
		/* overflow exit */
		if (h1 < 0)
			break;

		/* special for first digit */
		h = *dig;
		*dig = h % 10;

		/* quotient */
		h /= 10;
		/* check for valid results */
		if (h < 9)
		{
			/* make all prelimnary digits valid and write them out */
			/* putc ('Z',out); */
			for (j=0; j<stkptr ; j++)
				putc ((char)('0'+stack[j]),out);

			stack[0] = h;
			stkptr=1;

			/* write out decimal point */
			if (i == 1)
				putc ('.',out);
				
			/* if output is stdout */
			if (out == stdout)
				fflush(out);
		}
		else if (h==9)
		{
			if (stkptr == PRESTACK)
			{
				printf ("***Error: Prelimary stack overflow!\n\n");
				break;	/* leave for (i...) loop */
			}
			stack[stkptr++] = h;
		}
		else
		{
			/* putc ('X',out); */
			for (j=0; j<stkptr ; j++)
				putc ((char)(((stack[j]+1) % 10)+'0'),out);

			stack[0]=0;
			stkptr=1;
			/* if output is stdout */
			if (out == stdout)
				fflush(out);
		}
	}

	/* end with NL */
	putc ('\n',out);
	fflush(out);

	/* free memory */
	FREEMEM(dig);

	return sig-stkptr;
}
